/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// audiomsg.cpp Fri Jun 15 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "audiomsg.h"

AudioMsg::AudioMsg() // (const char* pcMessage)
{
    // initialize message with header
    strcpy(pcMessage, "WWFC");
}

AudioMsg::~AudioMsg()
{
}

Action* AudioMsg::copy() const
{
	return new AudioMsg(*this);
}

// return false if max message size has been reached
bool AudioMsg::addPlayer(const WorldPlayer& _wp)
{
    char pcBuf[MAX_SAY_MSG_SIZE]; // too big, but so what
    char team = 0;
    int age, number;
    float x, y, xvel, yvel;
    int bodydir, headdir;

    if(_wp.getTeam() == TEAM_US) team = 'u';
    else if(_wp.getTeam() == TEAM_THEM) team = 't';
    else team = '_'; // Error!

    age = _wp.getAge();
    number = _wp.getNumber();
    x       = _wp.getAbsPosition().getX();
    y       = _wp.getAbsPosition().getY();
    xvel    = _wp.getVelocity().getX();
    yvel    = _wp.getVelocity().getY();
    bodydir = (int)RAD_TO_DEG(_wp.getHeadDir());
    headdir = (int)RAD_TO_DEG(_wp.getBodyDir());

    sprintf(pcBuf, "((p %d %c %d) %.3f %.3f %.3f %.3f %d %d)", age, team, number,
            x, y, xvel, yvel, bodydir, headdir);

    // make sure we have room to add this message
    int iTotalLen = strlen(pcMessage);
    int iBufLen   = strlen(pcBuf);
    if((iTotalLen + iBufLen + 3) > MAX_SAY_MSG_SIZE)
        return false;

    // concatenate this message onto the end
    strncat(pcMessage, pcBuf, iBufLen);

    return true;
}

bool AudioMsg::addBall(const WorldBall& _wbBall)
{
    char pcBuf[MAX_SAY_MSG_SIZE];

    int age;
    float x, y, xvel, yvel;

    age  = _wbBall.getAge();
    x    = _wbBall.getAbsPosition().getX();
    y    = _wbBall.getAbsPosition().getY();
    xvel = _wbBall.getVelocity().getX();
    yvel = _wbBall.getVelocity().getY();

    sprintf(pcBuf, "((b %d) %.3f %.3f %.3f %.3f)", age, x, y, xvel, yvel);

    // make sure we have room to add this message
    int iTotalLen = strlen(pcMessage);
    int iBufLen   = strlen(pcBuf);
    if((iTotalLen + iBufLen + 3) > MAX_SAY_MSG_SIZE)
        return false;

    // concatenate this message onto the end
    strncat(pcMessage, pcBuf, iBufLen);

    return true;
}


ServerCommand* AudioMsg::translate() const
{
	// WARNING: creating new object, remember to delete!
	SayCommand* pscCmd = new SayCommand(pcMessage);
	
	return pscCmd;
}

ostream& AudioMsg::ostream_insert(ostream& _out) const
{
    _out << "AudioMsg";
    return _out;
}

ofstream& AudioMsg::ofstream_insert(ofstream& _fout) const
{
    return _fout;
}
