/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// coachcontrol.cpp Thu Jun 28 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "coachcontrol.h"

CoachControl::CoachControl(char* _pcTeamname, const char* _pcHostName, int _iPort, int _iReconnectNumber)
{
	// prepare the identity data and register with the server
	IdentityData idAboutMe;

	// team_name deleted by identitydata destructor
	// host information passed in from above, either from command line or as defaults
	idAboutMe.team_name = new char[strlen(_pcTeamname)];
	strcpy(idAboutMe.team_name, _pcTeamname);
	
	// TODO: this needs to be controlled by a command line
	idAboutMe.type = CLIENT_ONLINECOACH; // What about training coach?

	// host information passed in from above, either from command line or as defaults
	bdBody.init(idAboutMe, _pcHostName, _iPort, _iReconnectNumber);
	
	// insert the IdentityData into the WorldModel
	idAboutMe.worldmodel_insert(cwmWorld);
	
	// initialize behaviors and perceptions
	int i;
	for (i = 0; i < NUM_BEHAVIORS; i++)
	{
		pbeBehaviors[i] = NULL;
	}
	
/*	for (i = 0; i < NUM_PERCEPTIONS; i++)
	{
		pperPerceptions[i] = NULL;
	}
*/
}

CoachControl::~CoachControl()
{
	int i;
	for(i = 0; i < NUM_BEHAVIORS; i++)
	{
		if (pbeBehaviors != NULL)
			delete pbeBehaviors[i];
	}
/*	
	for(i = 0; i < NUM_PERCEPTIONS; i++)
	{
		if (pperPerceptions[i] != NULL) {
			delete pperPerceptions[i];
		}
	}
*/
}

void CoachControl::coach()
{
	// open log for writing
	char cFilename[80];
	snprintf(cFilename, 80, "%s%s%d%s", WORLD_LOG_FILENAME, cwmWorld.getTeamName(),
           0, WORLD_LOG_EXTENSION);
 	ofCoach.open(cFilename, ios::out);
 	
  	ofCoach << "<?xml version=\"1.0\"?>" << endl;
    ofCoach << "<!DOCTYPE playerlog SYSTEM \"playerlog.dtd\">" << endl;

    ofCoach << "<playerlog>" << endl;		
 	
 	EyeOnAction eoAct;
 	bdBody.execute(&eoAct);
	
//	Recommendation recFinal;
	
	int iLastTime;
	
	// while the client's body is connected to the server
	while(bdBody.isConnected())
	{
		iLastTime = cwmWorld.getTime();
		
	    // listen
	    listen();
		
		if(cwmWorld.getTime() != iLastTime)
	 		ofCoach << cwmWorld; // actually does quite a bit
	}
	
	// close log
	ofCoach << "</playerlog>" << endl;		
	ofCoach.close();

}

void CoachControl::listen()
{
	
	bdBody.senseWorld();
	SensorData* psdData;
	// while the body has sensordata
	// TODO: use the importance of the sensordata to triage
	//			 information
	while (bdBody.hasSensorData())
	{
		psdData = bdBody.getNextSensorData();

//		cwmWorld += *psdData;
		psdData->worldmodel_insert(cwmWorld);
		delete psdData;
		psdData = NULL;
	}
}
