/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// catchcommand.cpp Fri Jun 22 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "catchcommand.h"

#undef TRACE
#define TRACE(cmd)

CatchCommand::CatchCommand() : ServerCommand(1)
{
}

CatchCommand::CatchCommand(double _dDirRad) : ServerCommand(1)
{
	setDirection(_dDirRad);
}

CatchCommand::~CatchCommand()
{
}

Comm& CatchCommand::comm_insert(Comm& _cmConnection) const
{
    double dir_deg = RAD_TO_DEG( dDirRad );

    if (dir_deg < MINMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (catch " << dir_deg
              << "), normalizing to " << MINMOMENT );
        dir_deg = MINMOMENT;
    }
    else if (dir_deg > MAXMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (catch " << dir_deg
              << "), normalizing to " << MAXMOMENT );
        dir_deg = MAXMOMENT;
    }
    else if (! ( (dir_deg >= MINMOMENT - EPS) &&
                 (dir_deg <= MAXMOMENT + EPS) ) )
    {
        // NaN ?
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (catch " << dir_deg
              << "), dropping command completely" );
        return _cmConnection;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(catch %f)", dir_deg);
    if (_cmConnection.sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg);
    }
    else
    {
        ALERT("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }

	return _cmConnection;
}

void CatchCommand::setDirection(double _dDirRad)
{
	dDirRad = _dDirRad;
}

double CatchCommand::getDirection() const
{
	return dDirRad;
}
