/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// lowlevelcomm.h Thu Jun 14 2001
// Description:
//////////////////////////////////////////////////////////////////////

#ifndef _LOWLEVELCOMM_H_
#define _LOWLEVELCOMM_H_

#include "types.h"
#include "debug.h"
#include "errors.h"
#include "identitydata.h"
#include "datastore.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <signal.h>

class LowLevelComm
{
public: 
	LowLevelComm();
	virtual ~LowLevelComm();

  // initiate connection to server
  comm_error init(const char* hostname, int port);

  bool isServerAlive();
	void handleSignals(SensorDataStore &_store);
  comm_error sendToServer(char *msg);

  static int iWorldTime;

protected:
	virtual void finishAlarm() = 0;
	virtual void finishIo(SensorDataStore &_store) = 0;
  int        getTimeExpression();
  comm_error receiveFromServer(char *msg, long timeout_msec);
  comm_error createSocket (const char *host, const int port);
  comm_error blockSignals();
  comm_error unblockSignals();
  comm_error initSigHandlers();
  comm_error setAlarmInterval(int msec);
  void       removeSigHandlers();

  bool         serverOk;

private:

  ofstream debug_out;

  int          sd;
  struct       sockaddr_in server_addr;

  static sigset_t     alrmask;
  static sigset_t     iomask;
  static sigset_t     bothmask;
  static sig_atomic_t alarmFlag;
  static sig_atomic_t ioFlag;


  static void alarmHandler(int signo); // the static alarm handlers
  static void ioHandler   (int signo);

  static void alarmAction();    // executed on SIGALRM via alarmHandler()
  static void ioAction();       // executed on SIGIO via ioHandler()



};

#endif
