/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// turnneckcommand.cpp Fri Jun 22 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "turnneckcommand.h"

#undef TRACE
#define TRACE(cmd)

TurnNeckCommand::TurnNeckCommand() : ServerCommand(0)
{
}

TurnNeckCommand::TurnNeckCommand(double _dAngle) : ServerCommand(0)
{
	setAngle(_dAngle);
}

TurnNeckCommand::~TurnNeckCommand()
{
}

Comm& TurnNeckCommand::comm_insert(Comm& _cmConnection) const
{
    double angle_deg = RAD_TO_DEG( dAngle );

    if (angle_deg < MINNECKMOMENT - EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn_neck " << angle_deg
              << "), normalizing to " << MINNECKMOMENT );
        angle_deg = MINNECKMOMENT;
    }
    else if (angle_deg > MAXNECKMOMENT + EPS)
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn_neck " << angle_deg
              << "), normalizing to " << MAXNECKMOMENT );
        angle_deg = MAXNECKMOMENT;
    }
    else if (! ( (angle_deg >= MINNECKMOMENT - EPS) &&
                 (angle_deg <= MAXNECKMOMENT + EPS) ) )
    {
        TRACE("servertime_here"
              << " PlayerComm: agent tries to send (turn_neck " << angle_deg
              << "), dropping command completely" );
        return _cmConnection;
    }

    char msg [MAX_SERVER_MSG_SIZE];
    sprintf(msg, "(turn_neck %f)", angle_deg);
    if (_cmConnection.sendToServer(msg) == COMM_OK)
    {
        TRACE("PlayerComm: agent sent " << msg << endl);
    }
    else
    {
        ALERT("servertime_here"
              << " PlayerComm: agent cannot send " << msg << " to server" );
    }
	
    return _cmConnection;
}

void TurnNeckCommand::setAngle(double _dAngle)
{
	dAngle = _dAngle;
}

double TurnNeckCommand::getAngle() const
{
	return dAngle;
}
