/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// constants.h Thu Jun 7 2001
// Description: This file contains programmer defined contents for
//							The Robocup code
//////////////////////////////////////////////////////////////////////

#ifndef _CONSTANTS_H_
#define _CONSTANTS_H_

#include "types.h"
#include "vector2d.h"
#include "angle.h"
#include "debug.h"


// NOTE: eventually these constants will need to be cleaned out
//			 some of them may not be used in the project

/*
 *  Technical constants which are not in server.conf
 */

// default soccerserver host and port
#define DEFAULT_HOST "localhost"

// constants used to generate log filenames
#define WORLD_LOG_FILENAME "world"
#define WORLD_LOG_EXTENSION ".log\0"


// margin on top of SIMULATOR_STEP, before any messages will be missed
#define COMM_ALARM_MARGIN_USEC 5000

// minimum time interval between any two UDP-sends
#define COMM_MINIMUM_SEND_PAUSE_USEC 0

// Maximum cycles we wait before server is considered dead
#define COMM_MAX_DEAD_SERVER_CYCLES 50

// the message protocol we would like to use with the soccerserver (4.x or 5.x)
#define CLIENT_VERSION "5.00"

// size of center circle
#define CENTER_CIRCLE_R 9.15

// freekick forbidden area
#define FREE_KICK_FORBIDDEN_AREA 9.15

#define EPS 1e-10

/*
 *  Simulation/server related definitions
 */

// max length of server messages. Depending on implementation of sserver !
#define MAX_SERVER_MSG_SIZE 4096

// max length of say messages. Change this if value in server.conf has changed!
#define MAX_SAY_MSG_SIZE 512

/*
 * 	Constants needed by worldobjects, worldmodel, and perceptions
 */

#define UNKNOWN -1
#define ETERNAL 10000
#define MAXAGE	20
#define INFINITY 100000

#define NUM_TEAMMATES 10
#define MAX_PLAYER 11
#define NUM_PLAYERS 22
#define NUM_OTHER_PLAYERS 21
#define NUM_BEHAVIORS 20
//#define NUM_PERCEPTIONS 0
#define TIME_IN_HALF 3000

// constants for actions
#define TURN_WINDOW 0.25
#define INTERCEPT_DASH_POWER 80

// Distances/directions in a SensorData object may be invalid, i.e.
// we may only see the distance but not the direction etc.
// These constants are used to mark distances or directions as invalid
#define INVALID_DIST -10000.0
#define INVALID_DIR  -10000.0

// Max Size of SensorDataStore Object
#define DATA_STORE_SIZE 20


//constants for centering behavior
#define CENTERING_MULTIPLIER 80
#define ATTACKING_BONUS 20
#define MIDDLE_BONUS 25
#define DEFENSIVE_BONUS 30
#define PENALTY_AREA_BONUS 30

//constants for disperse behavior
#define DEFENSIVE_DISPERSE_MULT 30.0
#define OFFENSIVE_DISPERSE_DIST 10.0
#define DISPERSE_GRADE_MULT 10.0
#define DIS_AGE 6

//constants for backline behavior
#define OFFENSIVE_BALL_DIST 40.0
#define DEFENSIVE_BALL_DIST 30.0

// constants loaded from configuration file (see constants.cpp)
extern int HALF_TIME_IN_CYCLES;
extern double GOAL_WIDTH;
extern double PLAYER_SIZE;
extern double PLAYER_DECAY;
extern double PLAYER_RAND;
extern double PLAYER_WEIGHT;
extern double PLAYER_SPEED_MAX;
extern double STAMINA_MAX;
extern double STAMINA_INC_MAX;
extern double RECOVER_DEC_THR;
extern double RECOVER_DEC;
extern double RECOVER_MIN;
extern double EFFORT_DEC_THR;
extern double EFFORT_DEC;
extern double EFFORT_INC_THR;
extern double EFFORT_INC;
extern double EFFORT_MIN;
extern double HEAR_MAX;
extern double HEAR_INC;
extern double HEAR_DECAY;
extern double INERTIA_MOMENT;
extern int    SENSE_BODY_STEP;
extern double CATCHABLE_AREA_L;
extern double CATCHABLE_AREA_W;
extern double CATCH_PROBABILITY;
extern int    CATCH_BAN_CYCLE;
extern int    GOALIE_MAX_MOVES;
extern double BALL_SIZE;
extern double BALL_DECAY;
extern double BALL_RAND;
extern double BALL_WEIGHT;
extern double BALL_SPEED_MAX;
extern double WIND_FORCE;
extern double WIND_DIR;
extern double WIND_RAND;
extern double KICKABLE_MARGIN;
extern double KICKRANGE;
extern double CKICK_MARGIN;
extern double DASH_POWER_RATE;
extern double KICK_POWER_RATE;
extern double VISIBLE_ANGLE;
extern double AUDIO_CUT_DIST;
extern double QUANTIZE_STEP;
extern double QUANTIZE_STEP_L;
extern double MAXPOWER;
extern double MINPOWER;
extern double MAXMOMENT;
extern double MINMOMENT;
extern double MAXNECKMOMENT;
extern double MINNECKMOMENT;
extern double MAXNECKANG;
extern double MINNECKANG;
extern int    PORT;
extern int    COACH_PORT;
extern int    OLCOACH_PORT;
extern int    SAY_COACH_CNT_MAX;
extern int    SAY_COACH_MSG_SIZE;
extern int    SEND_VI_STEP;
extern int    SIMULATOR_STEP;
extern int    SEND_STEP;
extern int    RECV_STEP;
extern int    HALF_TIME;
extern int    SAY_MSG_SIZE;
extern bool   USE_OFFSIDE;
extern double OFFSIDE_ACTIVE_AREA;
extern bool   FORBID_KICK_OFF_OFFSIDE;
extern double OFFSIDE_KICK_MARGIN;
extern int    SLOW_DOWN_FACTOR;

extern Vector2d HOME_POSITIONS[MAX_PLAYER];
extern Vector2d KICKOFF_POSITIONS[MAX_PLAYER];

/*
 *  Server constants that are not in the server.conf
 */
// but should be?

const double UNUM_FAR_LENGTH= 20.0;
const double UNUM_TOOFAR_LENGTH= 40.0;
const double TEAM_FAR_LENGTH= 40.0;
const double TEAM_TOOFAR_LENGTH= 60.0;
const double VISIBLE_DISTANCE= 3.0;


/*
 *  Definitions for the field
 */

const double FIELD_LENGTH = 105.0;
const double FIELD_WIDTH = 68.0;

const double PENALTY_AREA_LENGTH = 16.5;
const double PENALTY_AREA_WIDTH = 40.32;

const double GOAL_AREA_LENGTH = 5.5;
const double GOAL_AREA_WIDTH = 18.32;

extern double GOAL_WIDTH;
const double GOAL_DEPTH = 2.44;

// functions for accessing constants
const Vector2d FLAGPOS(flag_type which);
bool IsLineHorizontal(line_type which);
bool IsLineVertical(line_type which);
const Vector2d GOALPOS(goal_type which);

void loadConfig(const char* _pcFileName);

#endif

