/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// centerperception.cpp Fri Jul 6 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "centerperception.h"

CenterPerception::CenterPerception(const WorldModel &_wmWorld) : Perception(_wmWorld)
{
}

CenterPerception::~CenterPerception()
{
}


Vector2d CenterPerception::getBallAbsPos()
{
//returns the absolute position of the ball
return wmWorld.getWorldBall().getAbsPosition();
}

bool CenterPerception::inAttackingThird(WorldObject _woObj)
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is greater than 1/6 Length because greater than that
	//is the attacking third of the field
	if ( vAbsPos.getX() > (FIELD_LENGTH / 6.0) )
		return true;
	else return false;}

bool CenterPerception::inMiddleThird(WorldObject _woObj)
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than 1/6 Length and greater than -1/6 Length
	//because that is the middle third of the field
	if ( (vAbsPos.getX() < (FIELD_LENGTH / 6.0)) && (vAbsPos.getX() > -(FIELD_LENGTH / 6.0)) )
		return true;
	else return false;}

bool CenterPerception::inDefensiveThird(WorldObject _woObj)
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than -1/6 Length because less than that
	//is the defensive third of the field
	if ( vAbsPos.getX() < -(FIELD_LENGTH / 6.0) )
		return true;
	else return false;
}

bool CenterPerception::inOurPenaltyArea(WorldObject _woObj)
{
  Vector2d vAbsPos = _woObj.getAbsPosition();

	return ( ( vAbsPos.getX() > - 0.5 * FIELD_LENGTH ) &&
	   ( vAbsPos.getX() < - 0.5 * FIELD_LENGTH + PENALTY_AREA_LENGTH ) &&
	   ( fabs( vAbsPos.getY() ) < 0.5 * PENALTY_AREA_WIDTH ) );
}

WorldBall CenterPerception::getBall()
{
	//returns the world ball
	return wmWorld.getWorldBall();
}


double CenterPerception::distToVirtCenter (double _dVirtCenter)
{
		return  (wmWorld.getClientPlayer().getAbsPosition().getY() - _dVirtCenter);
}


Angle CenterPerception::getBodyDir()
{
 	return ( wmWorld.getClientPlayer().getBodyDir() );
}

int CenterPerception::getBallAge() const
{
	return ( wmWorld.getWorldBall().getAge() );
}

bool CenterPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_PLAY_ON) ||
        (pmTemp == PLAY_MODE_DROP_BALL)
      ) return true;

    return false;
}
