/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// goaliepositionbehavior.cpp Mon Jul 23 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "goaliepositionbehavior.h"

#undef TRACE
#define TRACE(cmd)

GoaliePositionBehavior::GoaliePositionBehavior(const WorldModel& _wmWorld) : perGPP(_wmWorld)
{
}

GoaliePositionBehavior::~GoaliePositionBehavior()
{
}
bool GoaliePositionBehavior::isUseful()
{
    if(!perGPP.isActiveMode())
        return false;

    // he's positioned right where he should be, and the ball's in the penalty area :
    // catch & intercept ball should handle this
    if(perGPP.inBallCone() && perGPP.inOurPenaltyArea() && perGPP.ballInPenaltyArea())
        return false;

    // definately consider acting if we're outside the penalty area or ball cone
    if(!perGPP.inOurPenaltyArea() || !perGPP.inBallCone())
        return true;

    // if we're too far from our desired position (we're not chasing the ball, since we would have bailed by now)
    if(perGPP.relPos(perGPP.getDesiredPos()).getLength() > 15.0)
        return true;

    return false;
}

void GoaliePositionBehavior::evaluate(RecommendationList& recList)
{
    GoaliePositionRecommendation gprFinal;
    gprFinal.szBehaviorName = "goalieposition";

    RunAction raRun;
    double dGrade;

    Vector2d vDesiredPos = perGPP.getDesiredPos();

    // if we're not in the penalty area, we're screwed up... just run home
    if(!perGPP.inOurPenaltyArea())
    {
        raRun.setDirection( perGPP.relPos(vDesiredPos).getAngle() );
        raRun.setIntensity( 80 );

        // distance to desired position

        double dDesired = perGPP.relPos(vDesiredPos).getLength() ;
        dGrade = dDesired * 4.0; // tweaking factor
        // if(dGrade > 99)
        dGrade = 99; // use 99 because catch MUST execute when it wants to

        gprFinal.addAction(raRun);
        raRun.setDirection(0);
        gprFinal.addAction(raRun);
        gprFinal.setGrade(dGrade);
        recList.insert(gprFinal);
        TRACE("outside penalty area, run to: " << vDesiredPos);
        return;
    }

    // if we're not in the ball cone, run towards it
    if(!perGPP.inBallCone())
    {
        Angle aRunAngle = perGPP.relPos(vDesiredPos).getAngle();

        raRun.setDirection( aRunAngle );
        raRun.setIntensity( 70 );

        dGrade = perGPP.getDistToCone() * 4.0; // tweaking factor
        if(dGrade > 84) dGrade = 84;

        gprFinal.addAction(raRun);
        raRun.setDirection(0);
        gprFinal.addAction(raRun);
        gprFinal.setGrade( dGrade );

        TRACE("outside ball cone, run to " << vDesiredPos);

        recList.insert(gprFinal);
        return;
    }


/*
    if(perGPP.relPos(Vector2d(-FIELD_LENGTH/2, 0)).getLength() > 10.0)
    {
        if(perGPP.amIBehindBall())
        {
            raRun.setDirection( perGPP.relPos(vDesiredPos).getAngle() );
            raRun.setIntensity( -80 );
        }
        else
        {
            raRun.setDirection( perGPP.relPos(vDesiredPos).getAngle());
            raRun.setIntensity( 80 );
        }

        gprFinal.addAction(raRun);
        gprFinal.setGrade(86);
        recList.insert(gprFinal);
        return;
    }


    // if the ball is far away or we don't know much about it or the goalie is far from where he should be
    // then just go to the desired position near the center of the goal
    if(!perGPP.isBallNear() || perGPP.isBallOld() || !perGPP.inOurPenaltyArea())
    {
        raRun.setDirection( perGPP.relPos(vDesiredPos).getAngle() + PI );
        raRun.setIntensity( -80 );

        // distance to desired position

        double dDesired = perGPP.relPos(vDesiredPos).getLength() ;
        dGrade = dDesired * 4.0; // tweaking factor
        if(dGrade > 100) dGrade = 100;

        gprFinal.addAction(raRun);
        gprFinal.setGrade(dGrade);
        recList.insert(gprFinal);
        return;
    }

*/
    TRACE("In GoaliePosition with nothing to do");
    // if we're here, we are in ball cone.  ICept ball & catch should take control
    return;
}
