/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// goaliepositionperception.cpp Mon Jul 23 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "goaliepositionperception.h"

const int GoaliePositionPerception::OLD_AGE = 6;

GoaliePositionPerception::GoaliePositionPerception(const WorldModel& _wmWorld) : Perception(_wmWorld)
{
    vUpperGoalPost.setXY(-FIELD_LENGTH / 2, GOAL_WIDTH / 2);
    vLowerGoalPost.setXY(-FIELD_LENGTH / 2, -GOAL_WIDTH / 2);
}

GoaliePositionPerception::~GoaliePositionPerception()
{
}

bool GoaliePositionPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();

    // these are the only modes when we DO NOT want to worry about positioning
    if( (pmTemp == PLAY_MODE_GOALIE_CATCH_US) ||
        // (pmTemp == PLAY_MODE_FREE_KICK_US) || ComboLogic function handles this case
        (pmTemp == PLAY_MODE_BEFORE_KICK_OFF)
      ) return false;

    return true;
}

bool GoaliePositionPerception::inOurPenaltyArea() const
{
  Vector2d vAbsPos = wmWorld.getClientPlayer().getAbsPosition();

	return ( ( vAbsPos.getX() > - 0.5 * FIELD_LENGTH ) &&
	   ( vAbsPos.getX() < - 0.5 * FIELD_LENGTH + PENALTY_AREA_LENGTH ) &&
	   ( fabs( vAbsPos.getY() ) < 0.5 * PENALTY_AREA_WIDTH ) );
}

bool GoaliePositionPerception::ballInPenaltyArea() const
{
    Vector2d vAbsPos = wmWorld.getWorldBall().getAbsPosition();
	
	return ( ( vAbsPos.getX() > - 0.5 * FIELD_LENGTH ) &&
	   ( vAbsPos.getX() < - 0.5 * FIELD_LENGTH + PENALTY_AREA_LENGTH ) &&
	   ( fabs( vAbsPos.getY() ) < 0.5 * PENALTY_AREA_WIDTH ) );

}

bool GoaliePositionPerception::inBallCone() const
{
    Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();
    Vector2d vGoaliePos = wmWorld.getClientPlayer().getAbsPosition(); // Point (r, s)

    if(vGoaliePos.getX() > vBallPos.getX())
        return false;

    // slope of line to upper goal post
    double dUpperSlope = (vUpperGoalPost.getY() - vBallPos.getY()) / (vUpperGoalPost.getX() - vBallPos.getX());
    double dLowerSlope = (vLowerGoalPost.getY() - vBallPos.getY()) / (vLowerGoalPost.getX() - vBallPos.getX());

    // intercepts (b), used in y = mx+b form
    double dUpperIcept = vUpperGoalPost.getY() - dUpperSlope * vUpperGoalPost.getX();
    double dLowerIcept = vLowerGoalPost.getY() - dLowerSlope * vLowerGoalPost.getX();

    // Y coordinates, using y=mx+b, of cone border at goalie's X coordinate
    double dUpperY = dUpperSlope * vGoaliePos.getX() + dUpperIcept;
    double dLowerY = dLowerSlope * vGoaliePos.getX() + dLowerIcept;

    if( (vGoaliePos.getY() < dUpperY) &&
        (vGoaliePos.getY() > dLowerY)
      ) return true;

   return false;
}

Vector2d GoaliePositionPerception::getDesiredPos() const
{
    double dDesiredX = -FIELD_LENGTH / 2 + 6; // 6 should be a variable eventually

    Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();
    /* Vector2d vGoaliePos = wmWorld.getClientPlayer().getAbsPosition(); */

    // slope of line to upper goal post
    double dUpperSlope = (vUpperGoalPost.getY() - vBallPos.getY()) / (vUpperGoalPost.getX() - vBallPos.getX());
    double dLowerSlope = (vLowerGoalPost.getY() - vBallPos.getY()) / (vLowerGoalPost.getX() - vBallPos.getX());

    // intercepts (b), used in y = mx+b form; b = y - mx
    double dUpperIcept = vUpperGoalPost.getY() - dUpperSlope * vUpperGoalPost.getX();
    double dLowerIcept = vLowerGoalPost.getY() - dLowerSlope * vLowerGoalPost.getX();

    // Y coordinates, using y=mx+b, of desired position in front of goal
    double dUpperY = dUpperSlope * dDesiredX + dUpperIcept;
    double dLowerY = dLowerSlope * dDesiredX + dLowerIcept;

//    if(!(dUpperY > dLowerY))
//        cerr << "dUpperY is less then dLowerY!" << endl;

    // cone width at x = dDesiredDistFromGoal
    double dConeWidth = fabs(dUpperY - dLowerY);
    double dDesiredY = min(dLowerY, dUpperY) + dConeWidth / 2;

    if(dDesiredY > (GOAL_WIDTH/2.0+1)) dDesiredY = GOAL_WIDTH/2.0+1;
    if(dDesiredY < (-GOAL_WIDTH/2.0-1)) dDesiredY = -GOAL_WIDTH/2.0-1;

    return Vector2d( dDesiredX, dDesiredY );
}


Angle GoaliePositionPerception::getAngleToBall() const
{
    Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();

    return relPos(vBallPos).getAngle();

}


float GoaliePositionPerception::getDistToCone() const
{
    if(inBallCone()) return 0.0f;

    Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();
    Vector2d vGoaliePos = wmWorld.getClientPlayer().getAbsPosition(); // Point (r, s)

    // slope of line to upper goal post
    double dUpperSlope = (vUpperGoalPost.getY() - vBallPos.getY()) / (vUpperGoalPost.getX() - vBallPos.getX());
    double dLowerSlope = (vLowerGoalPost.getY() - vBallPos.getY()) / (vLowerGoalPost.getX() - vBallPos.getX());

    // intercepts (b), used in y = mx+b form
    double dUpperIcept = vUpperGoalPost.getY() - dUpperSlope * vUpperGoalPost.getX();
    double dLowerIcept = vLowerGoalPost.getY() - dLowerSlope * vLowerGoalPost.getX();

    // dist. from goalie to cone
    double dDistUpper = fabs(dUpperSlope*vGoaliePos.getX() - vGoaliePos.getY() + dUpperIcept)
                             / sqrt(dUpperSlope*dUpperSlope + 1);


    double dDistLower = fabs(dLowerSlope*vGoaliePos.getX() - vGoaliePos.getY() + dLowerIcept)
                             / sqrt(dLowerSlope*dLowerSlope + 1);

    return min(dDistUpper, dDistLower);
}


bool GoaliePositionPerception::isBallNear() const
{
	Vector2d vAbsPos = wmWorld.getWorldBall().getAbsPosition();
	
	//finds if the X coord is less than -1/6 Length because less than that
	//is the defensive third of the field
	if ( vAbsPos.getX() < -(FIELD_LENGTH / 6.0) )
		return true;
	
	return false;


}

bool GoaliePositionPerception::isBallOld() const
{
    return (wmWorld.getWorldBall().getAge() > OLD_AGE);
}

bool GoaliePositionPerception::amIBehindBall() const
{
    return wmWorld.getClientPlayer().getAbsPosition().getX() < wmWorld.getWorldBall().getAbsPosition().getX();
}

