/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// gotocornerperception.cpp Sat Aug 4 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "gotocornerperception.h"

GoToCornerPerception::GoToCornerPerception(const WorldModel &_wmWorld) : Perception(_wmWorld)
{
}

GoToCornerPerception::~GoToCornerPerception()
{
}

Angle GoToCornerPerception::angleToGoal() const
{
	WorldObject woGoal = wmWorld.getOtherGoal();
	Vector2d vRelPos = relPos(woGoal);
	
	return vRelPos.getAngle();
}

Angle GoToCornerPerception::angleToKick() const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	Angle aTop;
	Angle aBottom;
	Vector2d vPlayerPos = wmWorld.getClientPlayer().getAbsPosition();
	if ((vPlayerPos.getX() > 25.0) || (fabs(vPlayerPos.getY()) > (0.5*FIELD_WIDTH - 10.0)))
	{
		aTop = relPos(FLAGPOS(FLAG_RIGHT_TOP_30)).getAngle();
		aBottom = relPos(FLAGPOS(FLAG_RIGHT_BOTTOM_30)).getAngle();
	}
	else if (vPlayerPos.getX() < 10.0)
	{
		aTop = relPos(FLAGPOS(FLAG_TOP_RIGHT_20)).getAngle();
		aBottom = relPos(FLAGPOS(FLAG_BOTTOM_RIGHT_20)).getAngle();
	}
	else if (vPlayerPos.getX() <= 20.0)
	{
		aTop = relPos(FLAGPOS(FLAG_TOP_RIGHT_30)).getAngle();
		aBottom = relPos(FLAGPOS(FLAG_BOTTOM_RIGHT_30)).getAngle();
	}
    else
	{	
		aTop = relPos(FLAGPOS(FLAG_RIGHT_TOP_30)).getAngle();
		aBottom = relPos(FLAGPOS(FLAG_RIGHT_BOTTOM_30)).getAngle();
    }	
//	Angle aGoal = angleToGoal();
//	WorldBall wbBall = wmWorld.getWorldBall();
	
//	Vector2d vBallToTop = wbBall.getAbsPosition() - FLAGPOS(FLAG_TOP_RIGHT_30);
//	Vector2d vBallToBottom = wbBall.getAbsPosition() - FLAGPOS(FLAG_RIGHT_BOTTOM_30);
//	Vector2d vBallToPlayer = wbBall.getAbsPosition() - cwpClientPlayer.getAbsPosition();
	
//	if (cwpClientPlayer.getAbsPosition().getX() > FLAGPOS(FLAG_TOP_RIGHT_30).getX())
//	{
//		return aGoal;
//	}
//	else if (vBallToPlayer * vBallToTop > 0.0)
//	{
//		return relPos(FLAGPOS(FLAG_RIGHT_BOTTOM_30)).getAngle();
//	}
//	else if (vBallToPlayer * vBallToBottom > 0.0)
//	{
//		return relPos(FLAGPOS(FLAG_RIGHT_TOP_30)).getAngle();
//	}
	//else
	if ((aTop > PI_2) || (aTop < -PI_2))
	{
		return aBottom;
	}
	else if ((aBottom > PI_2) || (aBottom < -PI_2))
	{
		return aTop;
	}
	else if (cwpClientPlayer.getAbsPosition().getY() <  0.0)
	{
		return aTop;
	}
	else if (cwpClientPlayer.getAbsPosition().getY() >=  0.0)
	{
		return aBottom;
	}
	
	return Angle(0.0);

}

bool GoToCornerPerception::inTopHalf () const
{
  return (wmWorld.getClientPlayer ().getAbsPosition ().getY () < 0.0);
}

bool GoToCornerPerception::isOurGoalKick () const
{
  return wmWorld.getPlayMode () == PLAY_MODE_GOAL_KICK_US;
}

bool GoToCornerPerception::atBall() const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	WorldBall wbBall = wmWorld.getWorldBall();
//	Vector2d vPlayerVel = cwpClientPlayer.getVelocity();
//	Vector2d vBallVel = wbBall.getVelocity();
	
	double dRangeCoef = 1.0;
	//double dRangeCoef = 1.0 - norm(vPlayerVel + vBallVel) / (cwpClientPlayer.getMaxVelocity() + wbBall.getMaxVelocity());
	//bool f = norm(cwpClientPlayer.getAbsPosition() + vPlayerVel * PLAYER_DECAY - (wbBall.getAbsPosition() + vBallVel * BALL_DECAY)) < KICKRANGE;
	//return (dist(cwpClientPlayer.getAbsPosition() + vPlayerVel * PLAYER_DECAY, wbBall.getAbsPosition() + vBallVel * BALL_DECAY) < KICKRANGE);
	//return f;
	
	return (dist(cwpClientPlayer, wbBall) < (dRangeCoef * KICKRANGE));
	
}


bool GoToCornerPerception::ballMovingFaster()
{
	WorldBall wbBall = wmWorld.getWorldBall();
	Vector2d vCurrentVel = wbBall.getVelocity();
	
	if (vCurrentVel.getLength() < vLastVel.getLength())
	{
		vLastVel = vCurrentVel;
		return false;
	}
	
	vLastVel = vCurrentVel;
	
	return true;
}

bool GoToCornerPerception::willPassBall(double _dDashPower) const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	WorldBall wbBall = wmWorld.getWorldBall();
	
	double dCurrentDist = dist(cwpClientPlayer, wbBall);
	
	Vector2d  vAcceleration;
	vAcceleration.setRadAng((cwpClientPlayer.getEffort() * DASH_POWER_RATE * _dDashPower), (cwpClientPlayer.getBodyDir()));
	
	Vector2d vFuturePlayerPos = cwpClientPlayer.getAbsPosition() + cwpClientPlayer.getVelocity() * PLAYER_DECAY + vAcceleration;
	Vector2d vFutureBallPos = wbBall.getAbsPosition() + wbBall.getVelocity() * BALL_DECAY;
	
	double dFutureDist = norm(vFutureBallPos - vFuturePlayerPos);
	
	if (dFutureDist > (dCurrentDist + BALL_SIZE + PLAYER_SIZE))
		return true;
		
	return false;
}

bool GoToCornerPerception::ballAged() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	
	return (wbBall.getAge() > 0);
}
