
/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// goalkickbehavior.cpp Tue Jul 24 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "puntbehavior.h"

//#undef TRACE
//#define TRACE(cmd)

PuntBehavior::PuntBehavior(const WorldModel& _wmWorld) : perPunt(_wmWorld)
{
    iMoveCount = 0;
    iTimeCaught = 0;
    fSentTurn = false;
}

PuntBehavior::~PuntBehavior()
{
}

bool PuntBehavior::isUseful()
{

    if(perPunt.isPlayModeCatch())
    {
        TRACE(perPunt.getTime() << ": PLAY_MODE_GOALIE_CATCH_US");
				return true;
    }

    if(perPunt.isActiveMode() && perPunt.holdingBall())
        return true;


		iMoveCount = 0;
		iTimeCaught = perPunt.getTime();

    return false;
}


void PuntBehavior::evaluate(RecommendationList& recList)
{
    Recommendation recFinal;
    recFinal.szBehaviorName = "punt";

    // only the goalie can do this, and it must be done; force it
    recFinal.setGrade(100);

    // first move to a good location
    if(!perPunt.atPuntLocation() && (iMoveCount < GOALIE_MAX_MOVES))
    {
        MoveAction maMove(perPunt.getPuntLocation());
        recFinal.addAction(maMove);
        iMoveCount++;
        TRACE(perPunt.getTime() << ": Moving at time ");
				// iTimeCaught = perPunt.getTime();
    }
    else if ((perPunt.atPuntLocation() || iMoveCount >= GOALIE_MAX_MOVES)
						&& (perPunt.getTime() < iTimeCaught + 125))
		{
    		TurnAction taTurn;
				taTurn.setDirection(0);
				recFinal.addAction(taTurn);
				recFinal.addAction(taTurn);
				recFinal.addAction(taTurn);				
				TRACE(perPunt.getTime() << ": Stalling...");
		}
    else if(!perPunt.facingUpfield() && (perPunt.atPuntLocation() || (iMoveCount >= GOALIE_MAX_MOVES)))
    {
        TurnAction taTurn;
        taTurn.setDirection(perPunt.angleToKick());
        recFinal.addAction(taTurn);
        fSentTurn = true;
        TRACE(perPunt.getTime() << ": Turning at time ");
    }
    else
    {
        KickAction kaKick;
        kaKick.setIntensity(90);

        kaKick.setDirection( perPunt.angleToKick() );
        recFinal.addAction(kaKick);

        // reset state
        fSentTurn = false;
        // iMoveCount = 0;
        TRACE(perPunt.getTime() << ": Kicking at time ");
    }

		TRACE(perPunt.getTime() << ": iTimeCaught: " << iTimeCaught);
    recList.insert(recFinal);
}

