
/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// goalkickperception.cpp Tue Jul 24 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "puntperception.h"

#undef TRACE
#define TRACE(cmd)

PuntPerception::PuntPerception(const WorldModel& _wmWorld) : Perception(_wmWorld)
{
    vUpperCorner.setXY(-FIELD_LENGTH/2 + PENALTY_AREA_LENGTH, PENALTY_AREA_WIDTH/2);
    vLowerCorner.setXY(-FIELD_LENGTH/2 + PENALTY_AREA_LENGTH, -PENALTY_AREA_WIDTH/2);
}

PuntPerception::~PuntPerception()
{
}

bool PuntPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();

    if( pmTemp == PLAY_MODE_FREE_KICK_US )
        return true;

    if( (pmTemp == PLAY_MODE_GOALIE_CATCH_US) )
        return true;

    return false;
}

bool PuntPerception::isPlayModeCatch() const
{
    return wmWorld.getPlayMode() == PLAY_MODE_GOALIE_CATCH_US;
}

bool PuntPerception::sendMove() const // as in, send a move instead of a run
{
    play_mode pmTemp = wmWorld.getPlayMode();

    if( (pmTemp == PLAY_MODE_GOALIE_CATCH_US) )
        return true;

    return false;
}

Vector2d PuntPerception::closestTeammate() const
{
	WorldPlayer wpPlayer;
	ClientWorldPlayer cwpPlayer = wmWorld.getClientPlayer();

	double dDist;
	double dShortestDist = INFINITY;
	int iClosestJerseyNum = UNKNOWN;
	int i;
	int iClientJerseyNum = cwpPlayer.getNumber();
	
	for(i = 1; i <= MAX_PLAYER; i++)
	{
		wpPlayer = wmWorld.getOurPlayer(i);
		if(wpPlayer.getNumber() == iClientJerseyNum)
		    continue;
		if (wpPlayer.isKnown())
		{
      		dDist = dist(cwpPlayer.getAbsPosition(), wpPlayer.getAbsPosition());
      		if (dDist < dShortestDist)
            {
  			    dShortestDist = dDist;
      			iClosestJerseyNum = wpPlayer.getNumber();
  	    	}
      	}
	}
	
	if(iClosestJerseyNum == UNKNOWN) return Vector2d(0, 0);
	return wmWorld.getOurPlayer(iClosestJerseyNum).getAbsPosition();
}

double PuntPerception::distToClosestOpponent(Vector2d _vOrigin) const
{
    double dDist;
    double dShortestDist = INFINITY;

    for(int i=1; i <= MAX_PLAYER; ++i)
    {
        WorldPlayer wpPlayer = wmWorld.getOtherPlayer(i);
        if(!wpPlayer.isKnown()) continue;

        dDist = dist(_vOrigin, wpPlayer.getAbsPosition());
        if(dDist < dShortestDist)
            dShortestDist = dDist;
    }

    return dShortestDist;
}

Vector2d PuntPerception::getPuntLocation() const
{
// for now, determine which corner of penalty area we want to kick from

    double dDistUpper = distToClosestOpponent(vUpperCorner);
    double dDistLower = distToClosestOpponent(vLowerCorner);

    if(dDistUpper < dDistLower)
        return vUpperCorner + Vector2d(-1, -1);
    else
        return vLowerCorner + Vector2d(-1, 1);
}

Angle PuntPerception::angleToKick() const
{
		ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
		Vector2d vPlayerPos = cwpClientPlayer.getAbsPosition();
		
		Angle aKick;
		
		if (vPlayerPos.getY() < 0)
		{
				aKick = relPos(FLAGPOS(FLAG_RIGHT_TOP_30)).getAngle();
		}
		else
		{
				aKick = relPos(FLAGPOS(FLAG_RIGHT_BOTTOM_30)).getAngle();
		}
		
		return aKick;
}

bool PuntPerception::atPuntLocation() const
{
    return wmWorld.getClientPlayer().getAbsPosition() == getPuntLocation();
}

bool PuntPerception::inOurPenaltyArea() const
{
  Vector2d vAbsPos = wmWorld.getClientPlayer().getAbsPosition();

	return ( ( vAbsPos.getX() > - 0.5 * FIELD_LENGTH ) &&
	   ( vAbsPos.getX() < - 0.5 * FIELD_LENGTH + PENALTY_AREA_LENGTH ) &&
	   ( fabs( vAbsPos.getY() ) < 0.5 * PENALTY_AREA_WIDTH ) );
}

bool PuntPerception::holdingBall() const
{
    Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();
    Vector2d vGoaliePos = wmWorld.getClientPlayer().getAbsPosition();

    double dDist = dist(vBallPos, vGoaliePos);

    TRACE("Ball: " << vBallPos << " Goalie: " << vGoaliePos << " dist: " << dDist);

    return  dDist < 1.0;
}

bool PuntPerception::facingUpfield() const
{
    Angle aBodyDir = wmWorld.getClientPlayer().getBodyDir();

    if(fabs(aBodyDir) < PI_2) return true;

    return false;
}
