
/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// worldmodel.h Fri Jun 8 2001
// Description:
//////////////////////////////////////////////////////////////////////

#ifndef _WORLDMODEL_H_
#define _WORLDMODEL_H_

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>

#include "constants.h"
#include "angle.h"
#include "vector2d.h"
#include "debug.h"
#include "types.h"
#include "worldball.h"
#include "worldobject.h"
#include "worldplayer.h"
#include "clientworldplayer.h"
#include "sensor.h"
#include "baseworldmodel.h"

class WorldModel : public BaseWorldModel
{
public: 
	WorldModel();
	virtual ~WorldModel();
	
	// computeOrigin calculates the client players absolute position
	// on the field based on available flags and lines
	void computeOrigin(const SensorFlag _sfFlags[], const SensorLine _slLines[]);
	// getOrigin() returns the absolute position of the client player
	// on the field
	Vector2d getOrigin() const;
	ClientWorldPlayer getClientPlayer() const;
	// TODO: rewrite overloaded versions of modifyClientPlayer
	//			 to modify what you want to modify instead of returning
	//			 a pointer to the worldplayer for modification
	ClientWorldPlayer* modifyClientPlayer();
	// setClientPlayerNum() sets the jersey number and creates a worldclientplayer
	//	in the pwpOurTeam array
	void setClientPlayerNum(int _ClientPlayerNum);
	void setClientPlayer(WorldPlayer* _pwpClientPlayer);
	// TODO: add more generic mutators to WorldModel and BaseWorldModel
	//			 to allow the worldmodel to be changed with more than just sensorobjects
	void updateWorldBall(const SensorObject &_soBall);	
    void updateWorldBall(const WorldBall& _wbBall);	
	// updatePlayer(SensorPlayer) adds information
	// about the given SensorPlayer into the WorldModel
	void updatePlayer(const SensorPlayer &_spPlayer);
	
	// reimplementing function in BaseWorldModel to include ClientWorldPlayer
    // virtual void setTime(int _iTime);
    int getVisualTimeStamp() const;
    void setVisualTimeStamp(int _iLastVisualTimeStamp);

   	virtual ofstream& ofstream_insert(ofstream& _out);
 	
// 	ofstream& ofstream_insert(ofstream& _out);
// 	ifstream& ifstream_extract(ifstream& _in);

protected:
    virtual void age(); // Calls BaseWorldModel::age and then resets client's age to 0

private:
  ClientWorldPlayer* pwpClientPlayer;					// pointer to the client player for quick reference/editing
  int iLastVisualTimeStamp; // time when last received visual data
};

// WARNING: this function dumps binary data!!!
ofstream& operator<<(ofstream& _out, WorldModel &_wmWorld);
ifstream& operator>>(ifstream& _in, WorldModel &_wmWorld);

#endif
