/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// worldmovableobject.cpp Fri Jun 8 2001
// Description: This is the implementation of the WorldMovableObject
//							class.  WorldMovableObjects are WorldObjects that can
//							move.
//////////////////////////////////////////////////////////////////////

#include "worldmovableobject.h"

WorldMovableObject::WorldMovableObject(double _dMaxVelocity, double _dSize, const Vector2d &_vAbsPosition)
	: WorldObject( _vAbsPosition )
{
	setSize(_dSize);
	setConeTime(0);
	setMaxVelocity(_dMaxVelocity);
	setVelocity(Vector2d(0,0));
}

WorldMovableObject::WorldMovableObject(double _dMaxVelocity, double _dSize)
	: WorldObject()
{
	setSize(_dSize);
	setConeTime(0);
	setMaxVelocity(_dMaxVelocity);
	setVelocity(Vector2d(0,0));
}

void WorldMovableObject::update(const WorldMovableObject& _wmoObject)
{
    setAbsPosition(_wmoObject.getAbsPosition());
    setVelocity(_wmoObject.getVelocity());
    setAge(_wmoObject.getAge());
}

WorldMovableObject::~WorldMovableObject()
{
}

void WorldMovableObject::setMaxVelocity(double _dMaxVelocity)
{
	dMaxVelocity = _dMaxVelocity;
}

void WorldMovableObject::setSize(double _dSize)
{
	dSize = _dSize;
}

void WorldMovableObject::setConeTime(int _iConeTime)
{
	iConeTime = _iConeTime;
}

void WorldMovableObject::setVelocity(double dDistChange, double dDirChange)
{
 	// velocity in DistChange/timestep assuming one timestep per update
	// velocity = change in distance / change in time.  change in time = 1
	// WARNING: assumes dDirChange is value in radians
	vVelocity.setRadAng(dDistChange, Angle(dDirChange));
}

void WorldMovableObject::setVelocity(Vector2d _vVelocity)
{
    vVelocity = _vVelocity;
}

Vector2d WorldMovableObject::getVelocity() const
{
	return vVelocity;
}

double WorldMovableObject::getMaxVelocity()
{
	return dMaxVelocity;
}

double WorldMovableObject::getSize()
{
	return dSize;
}

int WorldMovableObject::getConeTime()
{
	return iConeTime;
}

void WorldMovableObject::increaseConeTime()
{
	iConeTime++;
}

ofstream& WorldMovableObject::ofstream_insert(ofstream &_fout)
{
    _fout << "    <worldmovableobject absx=\"" << getAbsPosition().getX() << "\" absy=\""
         << getAbsPosition().getY() << "\" xvel=\"" << getVelocity().getX() << "\" yvel=\""
         << getVelocity().getY() << "\" age=\"" << getAge() << "\"/>" << endl;

    return _fout;
}

ostream& WorldMovableObject::ostream_insert(ostream &_out)
{
    _out << "    <worldmovableobject absx=\"" << getAbsPosition().getX() << "\" absy=\""
         << getAbsPosition().getY() << "\" xvel=\"" << getVelocity().getX() << "\" yvel=\""
         << getVelocity().getY() << "\" age=\"" << getAge() << "\"/>" << endl;

    return _out;
}


ifstream& WorldMovableObject::ifstream_extract(ifstream &_in) {
    // TODO: Something

    return _in;
}
